#version 140

#include "../include/constants.vert"
#include "../include/clip_plane.vert"
#include "../include/quat_vec.vert"

in vec3 pos;
in vec2 uv;
in vec4 color;
in vec4 norm;
in vec4 tang;

uniform vec4 model_trans[MAX_MODEL_INSTANCES];
uniform vec4 model_quat[MAX_MODEL_INSTANCES];
uniform mat4 proj;

uniform vec3 ambColor[MAX_MODEL_INSTANCES];
uniform vec3 lightColor[MAX_MODEL_INSTANCES];
uniform vec3 lightDir[MAX_MODEL_INSTANCES];

uniform vec2 fadeout[MAX_MODEL_INSTANCES];

uniform vec4 color0;
uniform vec4 color1;

out vec2 uvOut;
out vec4 colOut;
out vec4 diffColOut;
out float fadeoutAmount;

out vec3 worldNormOut;

void main(void)
{
	vec3 vpos = pos;

	vec3 vnorm = quat_vec(model_quat[gl_InstanceID], norm.xyz);
	vec3 vtang = quat_vec(model_quat[gl_InstanceID], tang.xyz);

	// okay, so we have position, normal, tangent and bitangent

	vpos = quat_vec(model_quat[gl_InstanceID], vpos) * model_trans[gl_InstanceID].w +
		model_trans[gl_InstanceID].xyz;

	clip_vertex(vpos);
	vec3 vbitang = cross(vnorm, vtang) * tang.w;

	worldNormOut = vnorm*0.5 + vec3(0.5);

	float ln = dot(vnorm, lightDir[gl_InstanceID]);

	// half-lambert
	ln = ln*0.5 + 0.5;

	gl_Position = proj * vec4( vpos, 1.0 );
	uvOut = uv;

	colOut = color;

	vec2 fo = fadeout[gl_InstanceID];

	colOut *= mix(color0, color1, fo.y);

	diffColOut = colOut;

	vec3 final = lightColor[gl_InstanceID]*ln + ambColor[gl_InstanceID];

	colOut.xyz *= final*2.0;

	fadeoutAmount = fo.x;
}
