#version 140

#include "include/constants.vert"
#include "include/clip_plane.vert"
#include "include/quat_vec.vert"

in vec3 pos;
in vec2 uv;
in vec4 color;
in vec4 norm;
in vec4 tang;
in vec4 weights;
in vec4 bones;

uniform vec4 model_quat[MAX_MODEL_INSTANCES];
// w = uniform scale
uniform vec4 model_trans[MAX_MODEL_INSTANCES];

uniform mat4 proj;

uniform vec3 ambColor[MAX_MODEL_INSTANCES];
uniform vec3 lightColor[MAX_MODEL_INSTANCES];
uniform vec3 lightDir[MAX_MODEL_INSTANCES];

#ifdef USE_BILLBOARD
// xy = ofs, zw = scale; if zw is 0, treat as no scale
uniform vec4 billboard_offset[MAX_MODEL_INSTANCES];
#endif

out vec2 uvOut;
out vec4 colOut;

out vec3 worldNormOut;

void main(void)
{
	vec3 vpos = pos;

#ifdef USE_BILLBOARD
	vec3 vnorm = vec3(0.0, 0.0, 1.0);
	vec3 vtang = vec3(1.0, 0.0, 0.0);
#else
	vec3 vnorm = quat_vec(model_quat[gl_InstanceID], norm.xyz);
	vec3 vtang = quat_vec(model_quat[gl_InstanceID], tang.xyz);
#endif

	// okay, so we have position, normal, tangent and bitangent

	vpos = quat_vec(model_quat[gl_InstanceID], vpos) * model_trans[gl_InstanceID].w +
		model_trans[gl_InstanceID].xyz;

	clip_vertex(vpos);
	vec3 vbitang = cross(vnorm, vtang) * tang.w;

	worldNormOut = vnorm*0.5 + vec3(0.5);

	float ln = dot(vnorm, lightDir[gl_InstanceID]);

	// half-lambert
	ln = ln*0.5 + 0.5;

	gl_Position = proj * vec4( vpos, 1.0 );

	uvOut = uv;

#ifdef USE_BILLBOARD
	vec4 uvofs = billboard_offset[gl_InstanceID];

	if (uvofs.z == 0.0)
		uvofs.zw = vec2(1.0);

	uvOut.xy *= uvofs.zw;
	uvOut.xy += uvofs.xy;
#endif

	colOut = color;
	colOut.xyz *= (lightColor[gl_InstanceID]*ln + ambColor[gl_InstanceID])*2.0;
}
