#version 140

#include "include/constants.vert"
#include "include/clip_plane.vert"
#include "include/quat_vec.vert"

in vec3 pos;
in vec2 uv;
in vec4 color;
in vec4 norm;
in vec4 tang;
in vec4 weights;
in vec4 bones;

uniform vec4 model_quat[MAX_MODEL_INSTANCES];
// w = uniform scale
uniform vec4 model_trans[MAX_MODEL_INSTANCES];

uniform mat4 proj;

uniform vec3 ambColor[MAX_MODEL_INSTANCES];
uniform vec3 lightColor[MAX_MODEL_INSTANCES];
uniform vec3 lightDir[MAX_MODEL_INSTANCES];

// TODO: use uniform buffers, really...
uniform vec3 world_camera_position;

out vec2 uvOut;
out vec4 colOut;

out vec3 worldNormOut;
out vec3 worldDirOut;

void main(void)
{
	vec3 vpos = pos;
	vec3 vnorm = quat_vec(model_quat[gl_InstanceID], norm.xyz);
	vec3 vtang = quat_vec(model_quat[gl_InstanceID], tang.xyz);

	// okay, so we have position, normal, tangent and bitangent

	vpos = quat_vec(model_quat[gl_InstanceID], vpos) * model_trans[gl_InstanceID].w +
		model_trans[gl_InstanceID].xyz;

	clip_vertex(vpos);
	vec3 vbitang = cross(vnorm, vtang) * tang.w;

	worldNormOut = vnorm*0.5 + vec3(0.5);
	worldDirOut = vpos - world_camera_position;

	float ln = dot(vnorm, lightDir[gl_InstanceID]);

	// half-lambert
	ln = ln*0.5 + 0.5;

	gl_Position = proj * vec4( vpos, 1.0 );
	uvOut = uv;
	colOut = color;
	colOut.xyz *= (lightColor[gl_InstanceID]*ln + ambColor[gl_InstanceID])*2.0;
}
