#version 140

// old-school lightmapping
#ifdef GL_ES
precision highp float;
#endif

#include "include/fragData.frag"
#include "include/clip_plane.frag"
#include "include/srgb.frag"

//#define USE_GAMMA

// texture uvs
in vec2 uvOut;
// lightmap uvs
in vec2 uvOut2;

// diffuse color
uniform sampler2D tex;
// base color lightmap
uniform sampler2D tex2;

#ifndef NO_NORM_BUFFER
in vec3 worldNormOut;
#endif

#ifdef USE_GLOW_TEXTURE
// glow texture
uniform sampler2D tex3;
#endif

#ifdef USE_BASE_COLOR
uniform vec4 base_color;
#endif

#ifdef USE_DETAIL_TEXTURE
// usage in material:
//define "USE_DETAIL_TEXTURE"
//unit 4="detailTex" "path_to_detail_tex"
//constant detailUVScale 1.0 1.0

// detail texture
uniform sampler2D detailTex;

uniform vec2 detailUVScale;
#endif

vec4 sample_diffuse(vec2 uv)
{
	vec4 res = texture(tex, uv);
#ifdef USE_DETAIL_TEXTURE
	res.rgb *= texture(detailTex, uv * detailUVScale).rgb * 2.0;
#endif
	return res;
}

void main(void)
{
	clip_pixel();

/*	mat4 dither = mat4(
	0.0, 8.0, 2.0, 10.0,
	12.0, 4.0, 14.0, 6.0,
	3.0, 11.0, 1.0, 9.0,
	15.0, 7.0, 13.0, 5.0) / (16.0*128.0);*/

	vec4 b0 = unpack_lightmap(texture(tex2, uvOut2));

	// new: testing RGBM max 5 (ugly banding still but that's mostly because my rendertarget is 8 bit) + interpolation errors!
	//b0.xyz *= vec3(b0.w*6.0);

//#ifdef USE_GAMMA
	// unpack lightmap
	// note: *2.0 for old packed lightmaps => or we can also scale hdr lightmap output by half
//#endif

	vec4 diffuse;
	vec4 final;

#ifndef SUPERSAMPLE_DIFFUSE
	diffuse = sample_diffuse(uvOut);
#else
	vec2 samples[8]=vec2[8](
	vec2(-0.973062, -0.328256),
	vec2(0.110481, 0.712614),
	vec2(-0.818603, 0.638871),
	vec2(-0.018864, -0.385902),
	vec2(-0.430052, 0.142593),
	vec2(0.601915, 0.342594),
	vec2(0.554622, -0.808192),
	vec2(-0.474124, -0.883385)
	);

	vec4 sum = vec4(0.0);

	vec2 ofs = vec2(1.0) / textureSize(tex, 0);

	for (int i=0; i<8; i++)
	{
		vec4 sample = sample_diffuse(tex, uvOut + (samples[i])*ofs);
		sum += sample;
	}

	sum *= 1.0/8.0;

	diffuse = sum;

#endif

#ifdef USE_GAMMA
	diffuse.xyz = srgb_to_linear(diffuse.xyz);
#endif

#ifdef USE_BASE_COLOR
	diffuse *= base_color;
#endif

#ifndef NO_SPEC_BUFFER
	fragData3 = vec4(0.0);
#endif

#ifndef NO_NORM_BUFFER
	fragData2.xyz = worldNormOut;
#endif
	fragData1 = diffuse;

	// use diffuse alpha for potential transparency
	final.xyz = b0.xyz;

	// fake ambient
	//final.xyz = max(final.xyz, vec3(0.2));

	final.w = diffuse.w;

	// 2x overbright
	final.xyz *= diffuse.xyz * 2.0;

	// dither!
	//ivec2 iv = ivec2(gl_FragCoord.x, gl_FragCoord.y);
	//final.xyz += vec3(dither[iv.y & 3][iv.x & 3]);

#ifdef USE_GLOW_TEXTURE
#	ifdef GLOW_TEXTURE_MODULATE
	final.xyz *= texture(tex3, uvOut).xyz;
#	else
	final.xyz += texture(tex3, uvOut).xyz;
#	endif
#endif

#ifdef USE_GAMMA
	final.xyz = linear_to_srgb(final.xyz);
#endif

	fragColor = final;
}
