#version 140

// old-school lightmapping
#ifdef GL_ES
precision highp float;
#endif

#include "include/fragData.frag"
#include "include/clip_plane.frag"

//#define USE_GAMMA

// texture uvs
in vec2 uvOut;
// lightmap uvs
in vec2 uvOut2;

in mat3 tang_to_world;

// diffuse color
uniform sampler2D tex;
// base color lightmap
uniform sampler2D tex2;

#ifndef NO_NORM_BUFFER
in vec3 worldNormOut;
#endif

#ifndef NO_MIRROR
// mirror texture
uniform sampler2D tex3;
#endif

#ifdef REFRACT
// refraction texture
uniform sampler2D tex5;
// refraction depth
uniform sampler2D tex6;
#endif

#if defined(REFRACT) || defined(MIRROR_FRESNEL)
in vec3 worldDirOut;
#endif

uniform float mirror_scale;

#ifdef WATER
uniform float animTime0;
uniform float animTime1;
uniform float animTime2;
// dudv
uniform sampler2D tex4;

// 0.75
uniform float dudv_wave_scale;
// 100.0
uniform float dudv_sample_scale;
// 1.0
uniform float uv_scale;
#endif

#ifdef SUN_SPECULAR
uniform vec3 sun_direction;
uniform vec3 sun_color;
// 1.0
uniform float sun_specular_scale;
// 256
uniform float sun_specular_power;
// 0.5
uniform float sun_normal_strength;
#endif

#ifdef CUSTOM_FRESNEL
uniform float F0;
#endif

float pow5(float x)
{
	float tmp = x*x;
	return tmp*tmp*x;
}

#ifdef SUN_SPECULAR
vec3 ComputeSunSpecular(vec3 wdir, vec3 wnorm)
{
	wdir = normalize(wdir);
	vec3 refl = reflect(wdir, wnorm);
	vec3 res = sun_color;
	float dp = clamp(-dot(refl, sun_direction), 0.0, 1.0);
	float spec = pow(dp, sun_specular_power);
	return res * (spec * sun_specular_scale);
}
#endif

void main(void)
{
	clip_pixel();

	mat4 dither = mat4(
	0.0/16.0-0.5, 8.0/16.0-0.5, 2.0/16.0-0.5, 10.0/16.0-0.5,
	12.0/16.0-0.5, 4.0/16.0-0.5, 14.0/16.0-0.5, 6.0/16.0-0.5,
	3.0/16.0-0.5, 11.0/16.0-0.5, 1.0/16.0-0.5, 9.0/16.0-0.5,
	15.0/16.0-0.5, 7.0/16.0-0.5, 13.0/16.0-0.5, 5.0/16.0-0.5);

	vec4 b0 = unpack_lightmap(texture(tex2, uvOut2));

	// new: testing RGBM max 5 (ugly banding still but that's mostly because my rendertarget is 8 bit) + interpolation errors!
	//b0.xyz *= vec3(b0.w*6.0);

#ifdef USE_GAMMA
	// unpack lightmap
	b0.xyz = pow(b0.xyz, vec3(2.2));
#endif

	vec4 diffuse;
	vec4 final;

#ifdef WATER
	vec2 uv = uvOut;

	vec4 dudv = texture(tex4, vec2(0.5 + uv.x + animTime1, uv.y + animTime1));
	dudv *= 2.0;
	dudv -= 1.0;

	vec4 dudv2 = texture(tex4, vec2(0.75+uv.x + animTime1, uv.y - animTime1));
	dudv2 *= 2.0;
	dudv2 -= 1.0;

	//dudv = max(dudv, dudv2);
	dudv *= dudv2;

	// 0.75 = default
	dudv *= dudv_wave_scale;


	uv.y += animTime2;
	uv *= uv_scale;
	diffuse = texture(tex, uv + dudv.xy);
#else
	diffuse = texture(tex, uvOut);
#endif

#ifdef USE_GAMMA
	diffuse.xyz = pow(diffuse.xyz, vec3(2.2));
#endif

#ifndef NO_SPEC_BUFFER
	fragData3 = vec4(0.0);
#endif

#ifndef NO_NORM_BUFFER
	fragData2.xyz = worldNormOut;
#endif
	fragData1 = diffuse;

	// use diffuse alpha for potential transparency
	final.xyz = b0.xyz;

	// fake ambient
	//final.xyz = max(final.xyz, vec3(0.2));

	final.w = diffuse.w;

	// 2x overbright
	final.xyz *= diffuse.xyz * 2.0;

	{
		vec2 texSize = vec2(1.0) /
#ifndef NO_MIRROR
		vec2(textureSize(tex3, 0));
#else
		// assume refraction!
		vec2(textureSize(tex5, 0));
#endif
		// *0.5 because we use half-res
		vec2 mirrorUv = gl_FragCoord.xy * 0.5 * texSize;

#ifdef WATER
		vec2 mirrorUvOld = mirrorUv;
		// need a fixed scale here, but keep aspect ratio...
		mirrorUv += dudv.xy * dudv_sample_scale * vec2(1.0/(texSize.y*540.0/texSize.x), 1.0/540.0);
#endif

		vec3 mirror
#ifndef NO_MIRROR
		 = texture(tex3, mirrorUv).xyz
#endif
		;

#ifdef REFRACT
		vec3 refract = texture(tex5, mirrorUv).xyz;
#	ifdef WATER
		float refractDepth = texture(tex6, mirrorUv).x;
		//float refractDepthOrg = texture(tex6, mirrorUvOld).x;

		// slow but almost entirely eliminates boundary artifacts
		if (refractDepth >= 1.0)// || abs(1.0/refractDepth - 1.0/refractDepthOrg) > 0.001)
		{
#		ifndef NO_MIRROR
			mirror = texture(tex3, mirrorUvOld).xyz;
#		endif
			refract = texture(tex5, mirrorUvOld).xyz;
		}
#	endif

#	ifndef NO_MIRROR
		vec3 dir = normalize(worldDirOut);
		vec3 norm = normalize((worldNormOut - vec3(0.5)));
		//vec3 wn = tang_to_world * ((texture(tex4, uv+dudv.xy).xyz/*worldNormOut*/ - vec3(0.5))*2.0);
		//vec3 norm = normalize(wn);

		float NdotV = abs(dot(norm, dir));

		// Fresnel-Schlick
#	ifndef CUSTOM_FRESNEL
		float F0 = 0.05;
#	endif
		float fresnel = F0 + (1.0-F0)*pow5(max(0.0, 1.0 - NdotV));

#		ifdef WATER
		// scale diffuse gbuffer down because of lights
		fragData1.xyz = refract.xyz * 0.125;
		//fragData1.xyz *= fresnel;
#		endif

		// this is dumb, but never go past 100%
		// somehow scale reflection by lightmap...
 		//mirror *= min(b0.xyz * 0.5 + vec3(0.5), vec3(1.0));

		mirror = mix(refract, mirror, fresnel);
#	else
		mirror = refract;
#	endif

		//mirror.xyz = vec3(fresnel);
#endif

		float mscale = mirror_scale;

#ifdef MIRROR_FRESNEL
		vec3 dir = normalize(worldDirOut);
		vec3 norm = normalize((worldNormOut - vec3(0.5))*2.0);

		float NdotV = abs(dot(norm, dir));

		// Fresnel-Schlick
#	ifndef CUSTOM_FRESNEL
		float F0 = 0.05;
#	endif
		float fresnel = F0 + (1.0-F0)*pow5(max(0.0, 1.0 - NdotV));
		mscale *= fresnel;
#endif

#ifdef MIRROR_ADDITIVE
		final.xyz += mirror * mscale;
#else
		final.xyz = mix(final.xyz, mirror, mscale);
#endif

#ifdef SUN_SPECULAR
		vec3 wn = tang_to_world * ((mix(vec3(0.5, 0.5, 1.0), texture(tex4, uv+dudv.xy).xyz, sun_normal_strength) - vec3(0.5)));
		wn = normalize(wn);
		final.xyz += ComputeSunSpecular(worldDirOut, wn)/* * b0.xyz*/;
		//final.xyz = wn*0.5 + 0.5;
#endif
	}

	// dither!
	ivec2 iv = ivec2(gl_FragCoord.x, gl_FragCoord.y);
	final.xyz += vec3(dither[iv.y & 3][iv.x & 3] * (1.0/255.0));

#ifdef USE_GAMMA
	final.xyz = pow(final.xyz, vec3(1.0/2.2));
#endif

	fragColor = final;
}
