#version 140

#ifdef GL_ES
precision highp float;
#endif

out vec4 fragColor;

// depth
uniform sampler2D tex;
// normal - if 0, 0, 0 => backdrop
uniform sampler2D tex2;

// xy = positive 1/screen dims
// zw = projection matrix components
uniform vec4 ofs;

// these two specify frustum corners
uniform vec3 corner;
uniform vec3 cornerDelta;

uniform vec3 fogColor;
// fog range: x = near, y = far
uniform vec2 fogRange;
uniform float fogPower;

float LinearDepth(float d)
{
	// now with pre-baked denormalization and sign flip
	return ofs.w / (ofs.z + d);
}

void main()
{
	// Intel HD4000 crashes if this is const... tnx pta
	mat4 dither = mat4(
	0.0/16.0, 8.0/16.0, 2.0/16.0, 10.0/16.0,
	12.0/16.0, 4.0/16.0, 14.0/16.0, 6.0/16.0,
	3.0/16.0, 11.0/16.0, 1.0/16.0, 9.0/16.0,
	15.0/16.0, 7.0/16.0, 13.0/16.0, 5.0/16.0);

	vec2 uv = gl_FragCoord.xy * ofs.xy;

	float storedDepth = texture(tex, uv).x;
	vec3 storedNorm = texture(tex2, uv).xyz;

	fragColor.xyz = fogColor;

	// since we clear normal buffer after backdrop render, this hack avoid applying fog to backdrop
	if (/*storedDepth >= 1.0 ||*/ dot(storedNorm, storedNorm) == 0.0)
		fragColor.w = 0.0;
	else
	{
		float linear_depth = LinearDepth(storedDepth);
		vec3 view_pos = (corner + gl_FragCoord.xyz*cornerDelta)*linear_depth;

		float view_dist = length(view_pos);

		ivec2 iv = ivec2(mod(gl_FragCoord.xy, vec2(4.0)));
		float dith = dither[iv.y][iv.x] * (1.0/255.0);

		// compute fog blending factor
		fragColor.w = pow(clamp((view_dist - fogRange.x) / (fogRange.y - fogRange.x), 0.0, 1.0), fogPower) + dith;
	}

/*
	if ( ((int(gl_FragCoord.x) + int(gl_FragCoord.y)) & 1) != 0)
		fragColor = vec4(0.0, 0.0, 0.0, 1.0);*/
}
