#version 140

#ifdef GL_ES
precision highp float;
#endif

// a simple gradient based FXAA-style postprocess AA
// worse quality than true FXAA but faster, should probably
// call this PostAA instead to not confuse people... or "FXAA-like"

out vec4 fragColor;

// texture uvs
in vec2 uvOut;

// unfiltered input
uniform sampler2D tex;

// inverse framebuffer size
// zw = negative
uniform vec4 ofs;

// sample is reserved word... sigh
float luma(vec4 nsample)
{
	// perceptual
	return dot(nsample, vec4(0.3, 0.59, 0.11, 0.0));
}

void main(void)
{
	vec4 tnw = texture(tex, uvOut + ofs.zy);
	vec4 tn = texture(tex, vec2(uvOut.x, uvOut.y + ofs.y));
	vec4 tne = texture(tex, uvOut + ofs.xy);

	vec4 tw = texture(tex, vec2(uvOut.x + ofs.z, uvOut.y));
	vec4 source = texture(tex, uvOut);
	vec4 te = texture(tex, vec2(uvOut.x + ofs.x, uvOut.y));

	vec4 tsw = texture(tex, uvOut + ofs.zw);
	vec4 ts = texture(tex, vec2(uvOut.x, uvOut.y + ofs.w));
	vec4 tse = texture(tex, uvOut + ofs.xw);

	float nw = luma(tnw);
	float n = luma(tn);
	float ne = luma(tne);

	float w = luma(tw);
	float m = luma(source);
	float e = luma(te);

	float sw = luma(tsw);
	float s = luma(ts);
	float se = luma(tse);

	float maxLuma = max(nw, max(ne, max(sw, se)));
	maxLuma = max(maxLuma, max(n, max(s, max(e, w))));

	float maxLumaOut = maxLuma;
	maxLuma = max(m, maxLuma);

	float minLuma = min(nw, min(ne, min(sw, se)));
	minLuma = min(minLuma, min(n, min(s, min(e, w))));

	float minLumaOut = minLuma;
	minLuma = min(m, minLuma);

	float contrast = maxLuma - minLuma;

	fragColor = source;

	if (contrast >= 1.0/16.0)
	{
		// do AA
		vec2 dir;

		// copute Sobel gradient
		// still have problems with unfiltered isolated pixels though
		float gx = (ne + se + e*2.0) - (nw + sw + w*2.0);
		float gy = (ne + nw + n*2.0) - (sw + se + s*2.0);

		// we need to sample perpendicular to the gradient vector
		dir.x = -gy;
		dir.y = gx;

		// scale by contrast to avoid some artifacts
		dir *= 4.5 * (1.0-sqrt(contrast));

		// adjust for sampling (*inv_screen_dims)
		dir *= ofs.xy;

		float rcount = 1.0/6.0;

		vec2 hdir = dir * 0.5;

		vec4 res =
			texture(tex, uvOut - hdir) +
			source*2.0 +
			texture(tex, uvOut + hdir) +
			texture(tex, uvOut - dir) +
			texture(tex, uvOut + dir);

		// detect isolated pixels (rare)
		if (max(m - maxLumaOut, minLumaOut - m) >= 1.0/6.0)
		{
			res = tnw + tn + tne + tw + source*2.0 + te + tsw + ts + tse;
			rcount = 1.0/10.0;
		}

		fragColor = res * rcount;
	}
}
