#version 140

#ifdef GL_ES
precision highp float;
#endif

in vec2 uvOut;
in vec4 colOut;
//uniform sampler2D tex;

uniform vec4 global_time;
uniform vec4 global_frame;

out vec4 fragColor;

#define iGlobalTime (global_time.x)
#define iTime (global_time.x)

#define iResolution vec2(128.0)

vec2[] iChannelResolution = vec2[] (vec2(1024.0), vec2(1024.0));

uniform sampler2D tex, tex2;

#define iChannel0 tex
#define iChannel1 tex2

#define iFrame (int(global_frame.x))

#define iMouse vec2(0.0)

//-----------------------------------------------------------------------------


mat2 rotm(float c0, float s0)
{
	return mat2(c0, -s0, s0, c0);
}

vec3 compute_effect_2(vec2 uv)
{
	uv *= 2.0;
	uv -= vec2(1.0);

	float siTime = iTime + 100.0;

	float c0 = cos(siTime*0.5);
	float c1 = cos(siTime*c0*0.002*uv.x);
	float c2 = cos(siTime*c1*0.003*uv.y);
	float s0 = sin(siTime*0.5);
	float s1 = sin(siTime*s0*0.002+uv.x);
	float s2 = sin(siTime*s1*0.003+uv.y);

	mat2 rm = rotm(c0, s0);
	mat2 rm2 = rotm(c1, s1);
	mat2 rm3 = rotm(c2, s2);

	vec2 tuv = rm * uv * 2.0;
	tuv = rm2 * tuv;
	tuv = rm3 * tuv;


	float vl = length(tuv) * tuv.x;
	vec3 res = vec3(vl*tuv.x, vl*tuv.y, vl-tuv.x+tuv.y);
	res = abs(res);
	return res;
}

vec3 compute_effect(vec2 uv)
{
	vec3 res = compute_effect_2(compute_effect_2(uv).xy + vec2(0.5));

	res = pow(res, vec3(1.0/8.0));
	res = clamp(res, vec3(0.0), vec3(1.0));

	return res;
}

void mainImage(out vec4 fragColor, in vec2 fragCoord)
{
	vec2 uvofs = vec2(0.5)/iResolution.xy;
	vec4 ofs = vec4(uvofs.xy, 0.0, 0.0);
	vec2 uv = fragCoord/iResolution.xy;
	fragColor = vec4(compute_effect(uv+ofs.ww),1.0);
	fragColor += vec4(compute_effect(uv+ofs.xw),1.0);
	fragColor += vec4(compute_effect(uv+ofs.wy),1.0);
	fragColor += vec4(compute_effect(uv+ofs.xy),1.0);
	fragColor *= 0.25;
}

//-----------------------------------------------------------------------------

void main(void)
{
	vec2 fc = gl_FragCoord.xy;
	fc.y = iResolution.y - fc.y;
	mainImage(fragColor, fc);
}
