#version 140

#include "../include/constants.vert"
#include "../include/quat_vec.vert"

in vec3 pos;
in vec2 uv;
in vec4 weights;
in vec4 bones;

uniform vec4 model_trans[MAX_SKINNED_MODEL_INSTANCES];
uniform vec4 model_quat[MAX_SKINNED_MODEL_INSTANCES];
uniform mat4 proj;

// quaternions per bone
uniform vec4 skinQuat[MAX_SKINNED_MODEL_BONES];
// translations per bone, w = uniform scale
uniform vec4 skinTrans[MAX_SKINNED_MODEL_BONES];

uniform int bonesPerInstance;

out vec2 uvOut;

void main(void)
{
	vec3 vpos = vec3(0.0);

	int base_index = bonesPerInstance * gl_InstanceID;

	for (int i=0; i<4; i++)
	{
		int index = int(bones[i]) + base_index;
		float weight = weights[i];

		vec4 quat = skinQuat[index];
		vec4 trans = skinTrans[index];

		vpos += (quat_vec(quat, pos)*trans.w + trans.xyz) * weight;
	}

	vpos = quat_vec(model_quat[gl_InstanceID], vpos) * model_trans[gl_InstanceID].w +
		model_trans[gl_InstanceID].xyz;

	uvOut = uv;

	gl_Position = proj * vec4(vpos, 1.0);
}
