#version 140

#include "include/constants.vert"
#include "include/clip_plane.vert"
#include "include/quat_vec.vert"

in vec3 pos;
in vec2 uv;
in vec4 color;
in vec4 norm;
in vec4 tang;
in vec4 weights;
in vec4 bones;

uniform mat4 proj;

uniform vec4 model_trans[MAX_SKINNED_MODEL_INSTANCES];
uniform vec4 model_quat[MAX_SKINNED_MODEL_INSTANCES];

uniform vec3 ambColor[MAX_SKINNED_MODEL_INSTANCES];
uniform vec3 lightColor[MAX_SKINNED_MODEL_INSTANCES];
uniform vec3 lightDir[MAX_SKINNED_MODEL_INSTANCES];

// quaternions per bone
uniform vec4 skinQuat[MAX_SKINNED_MODEL_BONES];
// translations per bone, w = uniform scale
uniform vec4 skinTrans[MAX_SKINNED_MODEL_BONES];

uniform int bonesPerInstance;

out vec2 uvOut;
out vec4 colOut;

out vec3 worldNormOut;

void main(void)
{
	vec3 vpos = vec3(0.0);
	vec3 vnorm = vec3(0.0);
	vec3 vtang = vec3(0.0);

	int base_index = bonesPerInstance * gl_InstanceID;

	for (int i=0; i<4; i++)
	{
		int index = int(bones[i]) + base_index;
		float weight = weights[i];

		vec4 quat = skinQuat[index];
		vec4 trans = skinTrans[index];

		vpos += (quat_vec(quat, pos)*trans.w + trans.xyz) * weight;
		vnorm += quat_vec(quat, norm.xyz) * weight;
		vtang.xyz += quat_vec(quat, tang.xyz) * weight;
	}

	// okay, so we have position, normal, tangent and bitangent

	vpos = quat_vec(model_quat[gl_InstanceID], vpos) * model_trans[gl_InstanceID].w +
		model_trans[gl_InstanceID].xyz;

	clip_vertex(vpos);

	vnorm = quat_vec(model_quat[gl_InstanceID], vnorm);
	vtang = quat_vec(model_quat[gl_InstanceID], vtang);

	vec3 vbitang = cross(vnorm, vtang) * tang.w;

	worldNormOut = vnorm*0.5 + vec3(0.5);

	float ln = dot(vnorm, lightDir[gl_InstanceID]);

	// half-lambert (note: this is wrong as it's not squared...)
	ln = ln*0.5 + 0.5;

	gl_Position = proj * vec4( vpos, 1.0 );
	uvOut = uv;
	colOut = color;
	colOut.xyz *= (lightColor[gl_InstanceID]*ln + ambColor[gl_InstanceID])*2.0;
}
